@@ -2,16 +2,24 @@
 
 from __future__ import annotations
 
+from collections.abc import Mapping
 from typing import (
     TypeVar,
     Optional,
     NewType,
     Generic,
+    TYPE_CHECKING,
 )
 
 import attrs
 
 from caqtus.device.runtime import Device
+from caqtus.types.data import DataType, DataLabel
+from caqtus.utils.result import Success, Failure
+from .._name import DeviceName
+
+if TYPE_CHECKING:
+    from caqtus.shot_compilation import SequenceContext
 
 DeviceServerName = NewType("DeviceServerName", str)
 
@@ -42,5 +50,14 @@ class DeviceConfiguration(Generic[DeviceType]):
 
     remote_server: Optional[DeviceServerName] = attrs.field()
 
+    def get_data_schema(
+        self, name: DeviceName, sequence_context: "SequenceContext"
+    ) -> Success[Mapping[DataLabel, DataType]] | Failure[DataSchemaError]:
+        return Success({})
+
+
+class DataSchemaError(Exception):
+    """An error occurred while trying to get the data schema."""
+
 
 DeviceConfigType = TypeVar("DeviceConfigType", bound=DeviceConfiguration)
