@@ -6,10 +6,12 @@ from ._sequence_manager import (
 )
 from ._sequence_manager import run_sequence
 from .shot_timing import ShotTimer
+from ._logger import logger
 
 __all__ = [
     "SequenceManager",
     "ShotRetryConfig",
     "ShotTimer",
     "run_sequence",
+    "logger",
 ]
