@@ -212,9 +212,7 @@ class Experiment:
     def get_session_maker(self) -> StorageManager:
         return self.get_storage_manager()
 
-    def build_storage_manager[
-        T: StorageManager, **P
-    ](
+    def build_storage_manager[T: StorageManager, **P](
         self,
         backend_type: Callable[Concatenate[SerializerProtocol, P], T],
         *args: P.args,
@@ -238,9 +236,7 @@ class Experiment:
 
         return storage_backend_manager
 
-    def _build_storage_manager[
-        T: StorageManager, **P
-    ](
+    def _build_storage_manager[T: StorageManager, **P](
         self,
         backend_type: Callable[Concatenate[SerializerProtocol, P], T],
         *args: P.args,
@@ -338,6 +334,10 @@ class Experiment:
 
         The experiment server is used to run procedures on the experiment manager from a
         remote process.
+
+        If the environment variable `CAQTUS_BLOCKING_TASK_DURATION_WARNING` is set to
+        a float duration in seconds, a warning will be logged if a task doesn't yield to
+        the event loop for that duration during the execution of a sequence.
         """
 
         if platform.system() == "Windows":
@@ -401,6 +401,17 @@ class Experiment:
 
         return self.get_storage_manager().session()
 
+    def upgrade_database(self) -> None:
+        """Upgrade the database schema of the experiment to the latest version.
+
+        .. Warning::
+
+            If the database contains important data, it is strongly recommended to
+            back it up before running this function in case something goes wrong.
+        """
+
+        upgrade_database(self)
+
 
 def upgrade_database(experiment: Experiment) -> None:
     """Upgrade the database schema of the experiment to the latest version.
