@@ -1,10 +1,14 @@
 from __future__ import annotations
 
+import contextlib
 import functools
+import math
 import platform
-from collections.abc import Callable
+from collections.abc import Callable, Awaitable
+from typing import Never
 
 import anyio
+import anyio.abc
 import anyio.to_thread
 from PySide6 import QtGui, QtCore, QtWidgets
 from PySide6.QtCore import Qt
@@ -22,6 +26,11 @@ from .._logger import logger
 from .._parameter_tables_editor import ParameterNamespaceEditor
 from .._path_view import EditablePathHierarchyView
 from .._sequence_widget import SequenceWidget
+from .._sequence_widget.sequence_widget import (
+    _query_state_sync,
+    synchronize_editor_and_storage,
+    synchronize_sequence_widget,
+)
 from ..device_configuration_editors._configurations_editor import (
     DeviceConfigurationsDialog,
 )
@@ -43,6 +52,12 @@ class CondetrolWindowHandler:
         self.main_window.sequence_widget.sequence_start_requested.connect(
             self.start_sequence
         )
+        self.background_sequence_synchronization = BackgroundTask(
+            synchronize_sequence_widget,
+            100e-3,
+            self.main_window.sequence_widget,
+            self.session_maker,
+        )
 
     async def run_async(self) -> None:
         """Run the main window asynchronously."""
@@ -50,7 +65,7 @@ class CondetrolWindowHandler:
         async with self.task_group:
             self.task_group.start_soon(self.main_window.path_view.run_async)
             self.task_group.start_soon(self._monitor_global_parameters)
-            self.task_group.start_soon(self.main_window.sequence_widget.exec_async)
+            self.task_group.start_soon(self.background_sequence_synchronization.run)
 
     async def _monitor_global_parameters(self) -> None:
         while True:
@@ -58,7 +73,6 @@ class CondetrolWindowHandler:
                 parameters = await session.get_global_parameters()
             if parameters != self.main_window.global_parameters_editor.get_parameters():
                 self.main_window.global_parameters_editor.set_parameters(parameters)
-                self.main_window.sequence_widget.update_global_parameters(parameters)
             await anyio.sleep(0.2)
 
     def start_sequence(self, path: PureSequencePath):
@@ -91,6 +105,11 @@ class CondetrolWindowHandler:
         self.is_running_sequence = True
 
     async def _run_sequence(self, procedure: Procedure, sequence):
+        with self.background_sequence_synchronization.suspend():
+            async with self.session_maker.async_session() as session:
+                await synchronize_editor_and_storage(
+                    self.main_window.sequence_widget, session
+                )
         with procedure:
             try:
                 await anyio.to_thread.run_sync(procedure.start_sequence, sequence)
@@ -140,9 +159,7 @@ class CondetrolMainWindow(QtWidgets.QMainWindow, Ui_CondetrolMainWindow):
         self.global_parameters_editor = ParameterNamespaceEditor()
         self.connect_to_experiment_manager = connect_to_experiment_manager
         self.session_maker = session_maker
-        self.sequence_widget = SequenceWidget(
-            self.session_maker, extension.lane_extension, parent=self
-        )
+        self.sequence_widget = SequenceWidget(extension.lane_extension, parent=self)
         self.device_configurations_dialog = DeviceConfigurationsDialog(
             extension.device_extension, parent=self
         )
@@ -194,7 +211,9 @@ class CondetrolMainWindow(QtWidgets.QMainWindow, Ui_CondetrolMainWindow):
         )
 
     def set_edited_sequence(self, path: PureSequencePath):
-        self.sequence_widget.set_sequence(path)
+        with self.session_maker() as session:
+            state = _query_state_sync(path, session)
+        self.sequence_widget.set_fresh_state(state)
 
     def on_procedure_exception(self, exception: Exception):
         recoverable, non_recoverable = split_recoverable(exception)
@@ -280,7 +299,6 @@ class CondetrolMainWindow(QtWidgets.QMainWindow, Ui_CondetrolMainWindow):
         with self.session_maker() as session:
             session.set_global_parameters(parameters)
             logger.info(f"Global parameters written to storage: {parameters}")
-        self.sequence_widget.update_global_parameters(parameters)
 
     def signal_exception_while_running_sequence(self, exception: Exception):
         # This is a bit ugly because on_procedure_exception runs a dialog, which
@@ -304,3 +322,55 @@ class CondetrolMainWindow(QtWidgets.QMainWindow, Ui_CondetrolMainWindow):
             f"<p><i>caqtus-suite</i> version: {__version__}</p>"
             f"<p>Platform: {platform.platform()}</p>",
         )
+
+
+class BackgroundTask:
+    def __init__[
+        **P
+    ](
+        self,
+        task: Callable[P, Awaitable[None]],
+        sleep_time: float,
+        /,
+        *args: P.args,
+        **kwargs: P.kwargs,
+    ):
+        self.task = task
+        self._can_resume: anyio.Event | None = None
+        self._cancel_scope: anyio.CancelScope | None = None
+        self.sleep_time = sleep_time
+        self.args = args
+        self.kwargs = kwargs
+        self._task_group: anyio.abc.TaskGroup | None = None
+
+    async def run(self) -> Never:
+        """Run the background task indefinitely."""
+
+        async with anyio.create_task_group() as self._task_group:
+            await self.run_loop()
+            await anyio.sleep(math.inf)
+
+    async def run_loop(self) -> None:
+        with anyio.CancelScope() as self._cancel_scope:
+            while True:
+                await self.task(*self.args, **self.kwargs)
+                await anyio.sleep(self.sleep_time)
+
+    @contextlib.contextmanager
+    def suspend(self):
+        """Suspend the background task until the context manager exits.
+
+        When the context manager is entered, the background task is cancelled at the
+        next cancellation point.
+        When the context manager exits, a new background task is started.
+        """
+
+        if self._cancel_scope is None:
+            raise RuntimeError("The background task is not running.")
+        self._cancel_scope.cancel()
+        self._cancel_scope = None
+
+        yield
+
+        assert self._task_group is not None
+        self._task_group.start_soon(self.run_loop)
