@@ -19,17 +19,13 @@ from PySide6.QtWidgets import (
     QAbstractItemView,
 )
 
-from caqtus.gui._common.sequence_hierarchy import (
-    AsyncPathHierarchyView,
-)
+from caqtus.gui._common.sequence_hierarchy import AsyncPathHierarchyView
 from caqtus.gui.qtutil import temporary_widget
 from caqtus.session import (
     ExperimentSessionMaker,
     PureSequencePath,
     PathIsRootError,
     PathNotFoundError,
-)
-from caqtus.session import (
     PathIsSequenceError,
     PathHasChildrenError,
     State,
@@ -43,6 +39,7 @@ from caqtus.types.iteration import (
 from caqtus.types.timelane import TimeLanes
 from caqtus.types.variable_name import DottedVariableName
 from caqtus.utils.result import Failure, is_success, is_failure_type, unwrap
+from caqtus.utils.result._result import is_failure
 from ._icons import get_icon
 from .._common.waiting_widget import blocking_call
 
@@ -242,6 +239,8 @@ class EditablePathHierarchyView(AsyncPathHierarchyView):
             with self.session_maker() as session:
                 iterations = session.sequences.get_iteration_configuration(path)
                 time_lanes = session.sequences.get_time_lanes(path)
+                if is_failure(time_lanes):
+                    return
             if text.startswith(PureSequencePath._separator()):
                 if not PureSequencePath.is_valid_path(text):
                     QMessageBox.warning(  # type: ignore[reportCallIssue]
