@@ -1,10 +1,8 @@
-from .default_editor_creator import create_default_editor
 from .sequence_iteration_editor import SequenceIterationEditor, IterationEditorCreator
 from .steps_iteration_editor import StepsIterationEditor
 
 __all__ = [
     "SequenceIterationEditor",
     "IterationEditorCreator",
-    "create_default_editor",
     "StepsIterationEditor",
 ]
