@@ -3,7 +3,6 @@ from collections.abc import Callable, Set
 from typing import TypeVar, Generic, TypeAlias
 
 import caqtus.gui.qtutil.qabc as qabc
-from PySide6.QtCore import Signal
 from caqtus.types.iteration import IterationConfiguration
 from caqtus.types.variable_name import DottedVariableName
 
@@ -11,8 +10,6 @@ T = TypeVar("T", bound=IterationConfiguration)
 
 
 class SequenceIterationEditor(Generic[T], metaclass=qabc.QABCMeta):
-    iteration_edited = Signal(IterationConfiguration)
-
     @abc.abstractmethod
     def get_iteration(self) -> T:
         raise NotImplementedError
