@@ -3,22 +3,30 @@ from typing import Optional
 
 from PySide6 import QtCore
 from PySide6.QtCore import QRect
-from PySide6.QtGui import QKeySequence, QAction
-from PySide6.QtWidgets import QWidget, QVBoxLayout
+from PySide6.QtGui import QAction, QKeySequence
+from PySide6.QtWidgets import QVBoxLayout, QWidget
 
 from caqtus.device.sequencer.channel_commands import (
+    CalibratedAnalogMapping,
     ChannelOutput,
     Constant,
     DeviceTrigger,
     LaneValues,
-    CalibratedAnalogMapping,
+)
+from caqtus.device.sequencer.channel_commands.logic import (
+    AndGate,
+    NotGate,
+    OrGate,
+    XorGate,
 )
 from caqtus.device.sequencer.channel_commands.timing import Advance, BroadenLeft
-from caqtus.gui._common.NodeGraphQt import NodeGraph, BaseNode, NodesPaletteWidget
+from caqtus.gui._common.NodeGraphQt import BaseNode, NodeGraph, NodesPaletteWidget
+
 from ._analog_mapping_node import CalibratedAnalogMappingNode
 from ._constant_node import ConstantNode
 from ._device_trigger_node import DeviceTriggerNode
 from ._lane_node import LaneNode
+from ._not_gate_node import AndGateNode, NotGateNode, OrGateNode, XorGateNode
 from ._output_node import OutputNode
 from ._timing_nodes import AdvanceNode, BroadenLeftNode
 
@@ -35,6 +43,7 @@ class ChannelOutputEditor(QWidget):
         self.nodes_palette.set_category_label(
             "caqtus.sequencer_node.mapping", "Mapping"
         )
+        self.nodes_palette.set_category_label("caqtus.sequencer_node.logic", "Logic")
 
         layout = QVBoxLayout(self)
         layout.addWidget(self.graph.widget, 1)
@@ -57,6 +66,10 @@ class ChannelOutputGraph(NodeGraph):
         self.register_node(AdvanceNode)
         self.register_node(CalibratedAnalogMappingNode)
         self.register_node(BroadenLeftNode)
+        self.register_node(NotGateNode)
+        self.register_node(AndGateNode)
+        self.register_node(OrGateNode)
+        self.register_node(XorGateNode)
 
         self.output_node = OutputNode("out")
         self.add_node(self.output_node, selected=False, pos=[0, 0], push_undo=False)
@@ -165,6 +178,44 @@ class ChannelOutputGraph(NodeGraph):
         input_node.outputs()["out"].connect_to(node.input_port)
         return node
 
+    @build_node.register
+    def build_not_gate_node(self, not_gate: NotGate) -> NotGateNode:
+        node = NotGateNode()
+        self.add_node(node, selected=False, push_undo=False)
+        input_node = self.build_node(not_gate.input_)
+        input_node.outputs()["out"].connect_to(node.input_port)
+        return node
+
+    @build_node.register
+    def build_and_gate_node(self, and_gate: AndGate) -> AndGateNode:
+        node = AndGateNode()
+        self.add_node(node, selected=False, push_undo=False)
+        input_node_1 = self.build_node(and_gate.input_1)
+        input_node_2 = self.build_node(and_gate.input_2)
+        input_node_1.outputs()["out"].connect_to(node.input_port_1)
+        input_node_2.outputs()["out"].connect_to(node.input_port_2)
+        return node
+
+    @build_node.register
+    def build_or_gate_node(self, or_gate: OrGate) -> OrGateNode:
+        node = OrGateNode()
+        self.add_node(node, selected=False, push_undo=False)
+        input_node_1 = self.build_node(or_gate.input_1)
+        input_node_2 = self.build_node(or_gate.input_2)
+        input_node_1.outputs()["out"].connect_to(node.input_port_1)
+        input_node_2.outputs()["out"].connect_to(node.input_port_2)
+        return node
+
+    @build_node.register
+    def build_xor_gate_node(self, xor_gate: XorGate) -> XorGateNode:
+        node = XorGateNode()
+        self.add_node(node, selected=False, push_undo=False)
+        input_node_1 = self.build_node(xor_gate.input_1)
+        input_node_2 = self.build_node(xor_gate.input_2)
+        input_node_1.outputs()["out"].connect_to(node.input_port_1)
+        input_node_2.outputs()["out"].connect_to(node.input_port_2)
+        return node
+
 
 @functools.singledispatch
 def construct_output(node) -> ChannelOutput:
@@ -242,6 +293,72 @@ def construct_broaden_left(node: BroadenLeftNode) -> BroadenLeft:
     return BroadenLeft(width=width, input_=input_)
 
 
+@construct_output.register
+def construct_not_gate(node: NotGateNode) -> NotGate:
+    input_node = node.get_input_node()
+    if input_node is None:
+        raise MissingInputError(f"Not gate node {node.name()} must have an input node")
+    else:
+        input_ = construct_output(input_node)
+    return NotGate(input_=input_)
+
+
+@construct_output.register
+def construct_and_gate(node: AndGateNode) -> AndGate:
+    input_node_1, input_node_2 = node.get_input_nodes()
+    if input_node_1 is None:
+        raise MissingInputError(
+            f"And gate node {node.name()} must have an input node 1"
+        )
+    else:
+        input_1 = construct_output(input_node_1)
+
+    if input_node_2 is None:
+        raise MissingInputError(
+            f"And gate node {node.name()} must have an input node 2"
+        )
+    else:
+        input_2 = construct_output(input_node_2)
+
+    return AndGate(input_1=input_1, input_2=input_2)
+
+
+@construct_output.register
+def construct_or_gate(node: OrGateNode) -> OrGate:
+    input_node_1, input_node_2 = node.get_input_nodes()
+    if input_node_1 is None:
+        raise MissingInputError(f"Or gate node {node.name()} must have an input node 1")
+    else:
+        input_1 = construct_output(input_node_1)
+
+    if input_node_2 is None:
+        raise MissingInputError(f"Or gate node {node.name()} must have an input node 2")
+    else:
+        input_2 = construct_output(input_node_2)
+
+    return OrGate(input_1=input_1, input_2=input_2)
+
+
+@construct_output.register
+def construct_xor_gate(node: XorGateNode) -> XorGate:
+    input_node_1, input_node_2 = node.get_input_nodes()
+    if input_node_1 is None:
+        raise MissingInputError(
+            f"Xor gate node {node.name()} must have an input node 1"
+        )
+    else:
+        input_1 = construct_output(input_node_1)
+
+    if input_node_2 is None:
+        raise MissingInputError(
+            f"Xor gate node {node.name()} must have an input node 2"
+        )
+    else:
+        input_2 = construct_output(input_node_2)
+
+    return XorGate(input_1=input_1, input_2=input_2)
+
+
 class InvalidNodeConfigurationError(ValueError):
     pass
 
