@@ -6,9 +6,10 @@ from collections.abc import Mapping, Set, Iterable
 from typing import Protocol, Optional, Literal, TYPE_CHECKING
 
 import attrs
+import polars
 
 from caqtus.device import DeviceName, DeviceConfiguration
-from caqtus.types.data import DataLabel, Data
+from caqtus.types.data import DataLabel, Data, DataType
 from caqtus.types.iteration import IterationConfiguration, Unknown
 from caqtus.types.parameter import Parameter, ParameterNamespace, ParameterSchema
 from caqtus.types.timelane import TimeLanes
@@ -16,88 +17,25 @@ from caqtus.types.variable_name import DottedVariableName
 from caqtus.utils.result import Success, Failure, is_failure_type, is_failure
 from ._data_id import DataId
 from ._exception_summary import TracebackSummary
+from ._exceptions import (
+    PathIsSequenceError,
+    PathIsNotSequenceError,
+    InvalidStateTransitionError,
+    SequenceNotEditableError,
+    SequenceNotCrashedError,
+    PathNotFoundError,
+    PathHasChildrenError,
+)
+from ._exceptions import SequenceNotRunningError, SequenceNotLaunchedError
 from ._path import PureSequencePath
-from ._path_hierarchy import PathError, PathNotFoundError, PathHasChildrenError
 from ._shot_id import ShotId
 from ._state import State
+from ..shot_compilation import SequenceContext
 
 if TYPE_CHECKING:
     from ._experiment_session import ExperimentSession
 
 
-class PathIsSequenceError(PathError):
-    """Raised when a path is expected to be a sequence, but it is not."""
-
-    pass
-
-
-class PathIsNotSequenceError(PathError):
-    """Raised when a path is expected to not be a sequence, but it is."""
-
-    pass
-
-
-class DataNotFoundError(RuntimeError):
-    """Raised when data is not found in a shot."""
-
-    pass
-
-
-class SequenceStateError(RuntimeError):
-    """Raised when an invalid sequence state is encountered.
-
-    This error is raised when trying to perform an operation that is not allowed in the
-    current state, such as adding data to a sequence that is not in the RUNNING state.
-    """
-
-    pass
-
-
-class SequenceRunningError(SequenceStateError):
-    """Raised when trying to perform an invalid operation on a running sequence."""
-
-    pass
-
-
-class SequenceNotRunningError(SequenceStateError):
-    """Raised when trying to perform an invalid operation on a non-running sequence."""
-
-    pass
-
-
-class InvalidStateTransitionError(SequenceStateError):
-    """Raised when an invalid state transition is attempted.
-
-    This error is raised when trying to transition a sequence to an invalid state.
-    """
-
-    pass
-
-
-class SequenceNotEditableError(SequenceStateError):
-    """Raised when trying to edit a sequence that is not in the draft state."""
-
-    pass
-
-
-class SequenceNotLaunchedError(SequenceStateError):
-    """Raised when accessing information only available after launching a sequence."""
-
-    pass
-
-
-class SequenceNotCrashedError(SequenceStateError):
-    """Raised when trying to read the exceptions of a sequence that is not crashed."""
-
-    pass
-
-
-class ShotNotFoundError(RuntimeError):
-    """Raised when a shot is not found in a sequence."""
-
-    pass
-
-
 class SequenceCollection(Protocol):
     """A collection of sequences inside a session.
 
@@ -164,19 +102,20 @@ class SequenceCollection(Protocol):
         self,
         sequence: PureSequencePath,
         iteration_configuration: IterationConfiguration,
-    ) -> None:
-        """Set the iteration configuration for this sequence.
-
-        Raises:
-            PathNotFoundError: If the path doesn't exist.
-            PathIsNotSequenceError: If the path is not a sequence.
-            SequenceNotEditableError: If the sequence is not in DRAFT state.
-        """
+    ) -> (
+        None
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotEditableError]
+    ):
+        """Set the iteration configuration for this sequence."""
 
         raise NotImplementedError
 
     @abc.abstractmethod
-    def get_time_lanes(self, sequence_path: PureSequencePath) -> TimeLanes:
+    def get_time_lanes(
+        self, sequence_path: PureSequencePath
+    ) -> TimeLanes | Failure[PathNotFoundError] | Failure[PathIsNotSequenceError]:
         """Return a copy of the time lanes for this sequence."""
 
         raise NotImplementedError
@@ -184,28 +123,26 @@ class SequenceCollection(Protocol):
     @abc.abstractmethod
     def set_time_lanes(
         self, sequence_path: PureSequencePath, time_lanes: TimeLanes
-    ) -> None:
-        """Set the time lanes that define how a shot is run for this sequence.
-
-        Raises:
-            PathNotFoundError: If the path doesn't exist.
-            PathIsNotSequenceError: If the path is not a sequence.
-            SequenceNotEditableError: If the sequence is not in DRAFT state.
-        """
+    ) -> (
+        None
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotEditableError]
+    ):
+        """Set the time lanes that define how a shot is run for this sequence."""
 
         raise NotImplementedError
 
     @abc.abstractmethod
     def get_device_configurations(
         self, path: PureSequencePath
-    ) -> Mapping[DeviceName, DeviceConfiguration]:
-        """Get the device configurations that are used by this sequence.
-
-        Raises:
-            RuntimeError: If the sequence is in draft mode, since the device
-            configurations are only set once the sequence has entered the PREPARING
-            state.
-        """
+    ) -> (
+        Success[Mapping[DeviceName, DeviceConfiguration]]
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotLaunchedError]
+    ):
+        """Get the device configurations that are used by this sequence."""
 
         raise NotImplementedError
 
@@ -500,6 +437,52 @@ class SequenceCollection(Protocol):
         schema = iterations.get_parameter_schema(initial_values)
         return Success(schema)
 
+    def get_data_schema(
+        self, path: PureSequencePath
+    ) -> (
+        Success[Mapping[DataLabel, DataType]]
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotLaunchedError]
+    ):
+        device_configurations_result = self.get_device_configurations(path)
+        if is_failure(device_configurations_result):
+            return device_configurations_result
+        device_configurations = device_configurations_result.content()
+        parameter_schema_result = self.get_parameter_schema(path)
+        if is_failure(parameter_schema_result):
+            return parameter_schema_result
+        parameter_schema = parameter_schema_result.content()
+        time_lanes = self.get_time_lanes(path)
+        if is_failure(time_lanes):
+            return time_lanes
+        sequence_context = SequenceContext(
+            device_configurations, parameter_schema, time_lanes
+        )
+        schema = {}
+        for device_name, device_configuration in device_configurations.items():
+            device_schema = {
+                f"{device_name}\\{label}": data_type
+                for label, data_type in device_configuration.get_data_schema(
+                    device_name, sequence_context
+                )
+                .unwrap()
+                .items()
+            }
+            schema.update(device_schema)
+        return Success(schema)
+
+    @abc.abstractmethod
+    def scan(
+        self, path: PureSequencePath
+    ) -> (
+        Success[polars.LazyFrame]
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotLaunchedError]
+    ):
+        raise NotImplementedError
+
 
 @attrs.frozen
 class SequenceStats:
