@@ -3,6 +3,7 @@ import datetime
 from collections.abc import Mapping
 from typing import Protocol, Optional
 
+from caqtus.device import DeviceName, DeviceConfiguration
 from caqtus.types.data import DataLabel, Data
 from caqtus.types.iteration import IterationConfiguration
 from caqtus.types.parameter import Parameter, ParameterNamespace
@@ -11,16 +12,17 @@ from caqtus.types.variable_name import DottedVariableName
 from caqtus.utils.result import Success, Failure
 from .._data_id import DataId
 from .._exception_summary import TracebackSummary
-from .._path import PureSequencePath
-from .._path_hierarchy import PathNotFoundError
-from .._sequence_collection import (
-    PathIsNotSequenceError,
-    SequenceStats,
-    SequenceNotCrashedError,
+from .._exceptions import (
     SequenceNotRunningError,
-    InvalidStateTransitionError,
     SequenceNotLaunchedError,
+    PathIsNotSequenceError,
+    InvalidStateTransitionError,
+    SequenceNotEditableError,
+    SequenceNotCrashedError,
+    PathNotFoundError,
 )
+from .._path import PureSequencePath
+from .._sequence_collection import SequenceStats
 from .._shot_id import ShotId
 from .._state import State
 
@@ -77,7 +79,31 @@ class AsyncSequenceCollection(Protocol):
         raise NotImplementedError
 
     @abc.abstractmethod
-    async def get_time_lanes(self, path: PureSequencePath) -> TimeLanes:
+    async def set_iteration_configuration(
+        self, path: PureSequencePath, iteration_configuration: IterationConfiguration
+    ) -> (
+        None
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotEditableError]
+    ):
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    async def get_time_lanes(
+        self, path: PureSequencePath
+    ) -> TimeLanes | Failure[PathNotFoundError] | Failure[PathIsNotSequenceError]:
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    async def set_time_lanes(
+        self, path: PureSequencePath, time_lanes: TimeLanes
+    ) -> (
+        None
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotEditableError]
+    ):
         raise NotImplementedError
 
     @abc.abstractmethod
@@ -145,3 +171,16 @@ class AsyncSequenceCollection(Protocol):
         | Failure[SequenceNotRunningError]
     ):
         raise NotImplementedError
+
+    @abc.abstractmethod
+    async def get_device_configurations(
+        self, path: PureSequencePath
+    ) -> (
+        Success[Mapping[DeviceName, DeviceConfiguration]]
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotLaunchedError]
+    ):
+        """Get the device configurations that are used by this sequence."""
+
+        raise NotImplementedError
