@@ -23,7 +23,9 @@ from ._sequence_collection import (
     _is_sequence,
     _get_sequence_global_parameters,
     _get_time_lanes,
+    _set_time_lanes,
     _get_iteration_configuration,
+    _set_iteration_configuration,
     _get_shots,
     _get_shot_parameters,
     _get_shot_end_time,
@@ -33,21 +35,25 @@ from ._sequence_collection import (
     _get_exceptions,
     _create_shot,
     _reset_to_draft,
+    _get_device_configurations,
 )
 from ._serializer import SerializerProtocol
 from .._data_id import DataId
 from .._exception_summary import TracebackSummary
+from .._exceptions import (
+    PathIsSequenceError,
+    PathIsNotSequenceError,
+    InvalidStateTransitionError,
+    SequenceNotEditableError,
+    SequenceNotCrashedError,
+    PathNotFoundError,
+    PathIsRootError,
+)
+from .._exceptions import SequenceNotRunningError, SequenceNotLaunchedError
 from .._experiment_session import ExperimentSessionNotActiveError
 from .._path import PureSequencePath
-from .._path_hierarchy import PathNotFoundError, PathIsRootError
 from .._sequence_collection import (
-    PathIsSequenceError,
     SequenceStats,
-    PathIsNotSequenceError,
-    SequenceNotCrashedError,
-    SequenceNotRunningError,
-    InvalidStateTransitionError,
-    SequenceNotLaunchedError,
 )
 from .._shot_id import ShotId
 from ..async_session import (
@@ -55,6 +61,7 @@ from ..async_session import (
     AsyncPathHierarchy,
     AsyncSequenceCollection,
 )
+from ...device import DeviceName, DeviceConfiguration
 
 _T = TypeVar("_T")
 _P = ParamSpec("_P")
@@ -226,9 +233,21 @@ class AsyncSQLSequenceCollection(AsyncSequenceCollection):
     ):
         return await self._run_sync(_get_exceptions, path)
 
-    async def get_time_lanes(self, path: PureSequencePath) -> TimeLanes:
+    async def get_time_lanes(
+        self, path: PureSequencePath
+    ) -> TimeLanes | Failure[PathNotFoundError] | Failure[PathIsNotSequenceError]:
         return await self._run_sync(_get_time_lanes, path, self.serializer)
 
+    async def set_time_lanes(
+        self, path: PureSequencePath, time_lanes: TimeLanes
+    ) -> (
+        None
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotEditableError]
+    ):
+        return await self._run_sync(_set_time_lanes, path, time_lanes, self.serializer)
+
     async def get_global_parameters(
         self, path: PureSequencePath
     ) -> (
@@ -239,11 +258,33 @@ class AsyncSQLSequenceCollection(AsyncSequenceCollection):
     ):
         return await self._run_sync(_get_sequence_global_parameters, path)
 
+    async def get_device_configurations(
+        self, path: PureSequencePath
+    ) -> (
+        Success[Mapping[DeviceName, DeviceConfiguration]]
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotLaunchedError]
+    ):
+        return await self._run_sync(_get_device_configurations, path, self.serializer)
+
     async def get_iteration_configuration(
         self, path: PureSequencePath
     ) -> IterationConfiguration:
         return await self._run_sync(_get_iteration_configuration, path, self.serializer)
 
+    async def set_iteration_configuration(
+        self, path: PureSequencePath, iteration_configuration: IterationConfiguration
+    ) -> (
+        None
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotEditableError]
+    ):
+        return await self._run_sync(
+            _set_iteration_configuration, path, iteration_configuration, self.serializer
+        )
+
     async def get_shots(
         self, path: PureSequencePath
     ) -> (
