@@ -13,10 +13,12 @@ from typing import (
 import attrs
 import cattrs
 import numpy as np
+import polars
 import sqlalchemy.orm
 from sqlalchemy import func, select
 from sqlalchemy.orm import Session
 
+from caqtus.device import DeviceName, DeviceConfiguration
 from caqtus.types.data import Data
 from caqtus.types.data import is_data, DataLabel
 from caqtus.types.iteration import (
@@ -36,6 +38,8 @@ from caqtus.utils.result import (
     is_failure,
     unwrap,
 )
+from ._lazy_load import scan, structure_shot_sql_data
+from ._logger import logger
 from ._path_hierarchy import _query_path_model
 from ._path_table import SQLSequencePath
 from ._sequence_table import (
@@ -50,24 +54,24 @@ from ._serializer import SerializerProtocol
 from ._shot_tables import SQLShot, SQLShotParameter, SQLShotArray, SQLStructuredShotData
 from .._data_id import DataId
 from .._exception_summary import TracebackSummary
-from .._path import PureSequencePath
-from .._path_hierarchy import PathNotFoundError, PathHasChildrenError, PathIsRootError
-from .._sequence_collection import (
-    PathIsSequenceError,
-    PathIsNotSequenceError,
-    SequenceNotEditableError,
-    SequenceStats,
-    ShotNotFoundError,
-    DataNotFoundError,
-    SequenceNotCrashedError,
-    InvalidStateTransitionError,
+from .._exceptions import (
     SequenceNotRunningError,
     SequenceNotLaunchedError,
+    PathIsSequenceError,
+    PathIsNotSequenceError,
+    DataNotFoundError,
+    InvalidStateTransitionError,
+    SequenceNotEditableError,
+    SequenceNotCrashedError,
+    ShotNotFoundError,
+    PathNotFoundError,
+    PathIsRootError,
+    PathHasChildrenError,
 )
-from .._sequence_collection import SequenceCollection
+from .._path import PureSequencePath
+from .._sequence_collection import SequenceCollection, SequenceStats
 from .._shot_id import ShotId
 from .._state import State
-from ...device import DeviceName, DeviceConfiguration
 
 if TYPE_CHECKING:
     from ._experiment_session import SQLExperimentSession
@@ -193,17 +197,14 @@ class SQLSequenceCollection(SequenceCollection):
         self,
         sequence: PureSequencePath,
         iteration_configuration: IterationConfiguration,
-    ) -> None:
-        sequence_model = unwrap(self._query_sequence_model(sequence))
-        if not sequence_model.state.is_editable():
-            raise SequenceNotEditableError(sequence)
-        iteration_content = self.serializer.dump_sequence_iteration(
-            iteration_configuration
-        )
-        sequence_model.iteration.content = iteration_content
-        expected_number_shots = iteration_configuration.expected_number_shots()
-        sequence_model.expected_number_of_shots = _convert_from_unknown(
-            expected_number_shots
+    ) -> (
+        None
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotEditableError]
+    ):
+        return _set_iteration_configuration(
+            self._get_sql_session(), sequence, iteration_configuration, self.serializer
         )
 
     def create(
@@ -251,16 +252,22 @@ class SQLSequenceCollection(SequenceCollection):
     def serialize_time_lanes(self, time_lanes: TimeLanes) -> serialization.JSON:
         return self.serializer.unstructure_time_lanes(time_lanes)
 
-    def get_time_lanes(self, sequence_path: PureSequencePath) -> TimeLanes:
+    def get_time_lanes(
+        self, sequence_path: PureSequencePath
+    ) -> TimeLanes | Failure[PathNotFoundError] | Failure[PathIsNotSequenceError]:
         return _get_time_lanes(self._get_sql_session(), sequence_path, self.serializer)
 
     def set_time_lanes(
         self, sequence_path: PureSequencePath, time_lanes: TimeLanes
-    ) -> None:
-        sequence_model = unwrap(self._query_sequence_model(sequence_path))
-        if not sequence_model.state.is_editable():
-            raise SequenceNotEditableError(sequence_path)
-        sequence_model.time_lanes.content = self.serialize_time_lanes(time_lanes)
+    ) -> (
+        None
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotEditableError]
+    ):
+        return _set_time_lanes(
+            self._get_sql_session(), sequence_path, time_lanes, self.serializer
+        )
 
     def get_state(
         self, path: PureSequencePath
@@ -471,19 +478,15 @@ class SQLSequenceCollection(SequenceCollection):
 
     def get_device_configurations(
         self, path: PureSequencePath
-    ) -> dict[DeviceName, DeviceConfiguration]:
-        sequence = unwrap(self._query_sequence_model(path))
-        if sequence.state == State.DRAFT:
-            raise RuntimeError("Sequence has not been prepared yet")
-
-        device_configurations = {}
-
-        for device_configuration in sequence.device_configurations:
-            constructed = self.serializer.load_device_configuration(
-                device_configuration.device_type, device_configuration.content
-            )
-            device_configurations[device_configuration.name] = constructed
-        return device_configurations
+    ) -> (
+        Success[dict[DeviceName, DeviceConfiguration]]
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotLaunchedError]
+    ):
+        return _get_device_configurations(
+            self._get_sql_session(), path, self.serializer
+        )
 
     def get_stats(
         self, path: PureSequencePath
@@ -562,6 +565,41 @@ class SQLSequenceCollection(SequenceCollection):
         result = self._get_sql_session().execute(stmt).scalars().all()
         return (PureSequencePath(row.path) for row in result)
 
+    def scan(
+        self, path: PureSequencePath
+    ) -> (
+        Success[polars.LazyFrame]
+        | Failure[PathNotFoundError]
+        | Failure[PathIsNotSequenceError]
+        | Failure[SequenceNotLaunchedError]
+    ):
+        session = self._get_sql_session()
+        sequence_model_result = _query_sequence_model(session, path)
+        if is_failure(sequence_model_result):
+            return sequence_model_result
+        sequence_model = sequence_model_result.content()
+        parameter_schema_result = self.get_parameter_schema(path)
+        assert not is_failure_type(
+            parameter_schema_result, (PathNotFoundError, PathIsNotSequenceError)
+        )
+        parameter_schema = parameter_schema_result.content()
+        if sequence_model.state == State.DRAFT:
+            return Failure(SequenceNotLaunchedError(path))
+        data_schema_result = self.get_data_schema(path)
+        assert not is_failure_type(
+            data_schema_result,
+            (PathNotFoundError, PathIsNotSequenceError, SequenceNotLaunchedError),
+        )
+        data_schema = data_schema_result.content()
+        return Success(
+            scan(
+                session,
+                sequence_model,
+                {str(k): v for k, v in parameter_schema.items()},
+                {str(k): v for k, v in data_schema.items()},
+            )
+        )
+
     def _query_path_model(
         self, path: PureSequencePath
     ) -> (
@@ -746,13 +784,61 @@ def _get_iteration_configuration(
     )
 
 
+def _set_iteration_configuration(
+    session: Session,
+    sequence: PureSequencePath,
+    iteration_configuration: IterationConfiguration,
+    serializer: SerializerProtocol,
+) -> (
+    None
+    | Failure[PathNotFoundError]
+    | Failure[PathIsNotSequenceError]
+    | Failure[SequenceNotEditableError]
+):
+    sequence_model_result = _query_sequence_model(session, sequence)
+    if is_failure(sequence_model_result):
+        return sequence_model_result
+    sequence_model = sequence_model_result.content()
+    if not sequence_model.state.is_editable():
+        raise SequenceNotEditableError(sequence)
+    iteration_content = serializer.dump_sequence_iteration(iteration_configuration)
+    sequence_model.iteration.content = iteration_content
+    expected_number_shots = iteration_configuration.expected_number_shots()
+    sequence_model.expected_number_of_shots = _convert_from_unknown(
+        expected_number_shots
+    )
+
+
 def _get_time_lanes(
     session: Session, sequence_path: PureSequencePath, serializer: SerializerProtocol
-) -> TimeLanes:
-    sequence_model = unwrap(_query_sequence_model(session, sequence_path))
+) -> TimeLanes | Failure[PathNotFoundError] | Failure[PathIsNotSequenceError]:
+    sequence_model_result = _query_sequence_model(session, sequence_path)
+    if is_failure(sequence_model_result):
+        return sequence_model_result
+    sequence_model = sequence_model_result.content()
     return serializer.structure_time_lanes(sequence_model.time_lanes.content)
 
 
+def _set_time_lanes(
+    session: Session,
+    sequence_path: PureSequencePath,
+    time_lanes: TimeLanes,
+    serializer: SerializerProtocol,
+) -> (
+    None
+    | Failure[PathNotFoundError]
+    | Failure[PathIsNotSequenceError]
+    | Failure[SequenceNotEditableError]
+):
+    sequence_model_result = _query_sequence_model(session, sequence_path)
+    if is_failure(sequence_model_result):
+        return sequence_model_result
+    sequence_model = sequence_model_result.content()
+    if not sequence_model.state.is_editable():
+        return Failure(SequenceNotEditableError(sequence_path))
+    sequence_model.time_lanes.content = serializer.unstructure_time_lanes(time_lanes)
+
+
 def _get_shots(
     session: Session, path: PureSequencePath
 ) -> (
@@ -843,11 +929,9 @@ def _get_all_shot_data(
     structured_data = shot_model.structured_data
     result = {}
     for array in arrays:
-        result[array.label] = np.frombuffer(array.bytes_, dtype=array.dtype).reshape(
-            array.shape
-        )
+        result[array.label] = structure_shot_sql_data(array)
     for data in structured_data:
-        result[data.label] = data.content
+        result[data.label] = structure_shot_sql_data(data)
     return result
 
 
@@ -886,14 +970,14 @@ def _get_shot_start_time(
     session: Session, path: PureSequencePath, shot_index: int
 ) -> datetime.datetime:
     shot_model = unwrap(_query_shot_model(session, path, shot_index))
-    return shot_model.start_time.replace(tzinfo=datetime.timezone.utc)
+    return shot_model.get_start_time()
 
 
 def _get_shot_end_time(
     session: Session, path: PureSequencePath, shot_index: int
 ) -> datetime.datetime:
     shot_model = unwrap(_query_shot_model(session, path, shot_index))
-    return shot_model.end_time.replace(tzinfo=datetime.timezone.utc)
+    return shot_model.get_end_time()
 
 
 def _query_data_model(
@@ -1012,6 +1096,49 @@ def _query_shot_model(
                 assert_never(sequence_model_result)
 
 
+def _get_device_configurations(
+    session: Session, path: PureSequencePath, serializer: SerializerProtocol
+) -> (
+    Success[dict[DeviceName, DeviceConfiguration]]
+    | Failure[PathNotFoundError]
+    | Failure[PathIsNotSequenceError]
+    | Failure[SequenceNotLaunchedError]
+):
+    sequence_result = _query_sequence_model(session, path)
+    if is_failure(sequence_result):
+        return sequence_result
+    sequence = sequence_result.content()
+    if sequence.state == State.DRAFT:
+        return Failure(
+            SequenceNotLaunchedError(f"Sequence at {path} is in DRAFT state")
+        )
+
+    device_configurations = {}
+
+    for device_configuration in sequence.device_configurations:
+        try:
+            constructed = serializer.load_device_configuration(
+                device_configuration.device_type, device_configuration.content
+            )
+        except Exception as e:
+            logger.error(
+                "Could not load device configuration for %s\n%r",
+                device_configuration,
+                device_configuration.content,
+            )
+            raise DeserializationError(
+                f"Failed to load device configuration for "
+                f"<{device_configuration.name}> with tag "
+                f"<{device_configuration.device_type}>"
+            ) from e
+        device_configurations[device_configuration.name] = constructed
+    return Success(device_configurations)
+
+
+class DeserializationError(Exception):
+    pass
+
+
 def _reset_to_draft(
     session: Session, path: PureSequencePath
 ) -> (
@@ -1049,7 +1176,7 @@ def _reset_to_draft(
 
 
 def serialize_data(
-    data: Mapping[DataLabel, Data]
+    data: Mapping[DataLabel, Data],
 ) -> tuple[list[SQLShotArray], list[SQLStructuredShotData]]:
     arrays = []
     structured_data = []
