@@ -1,15 +1,17 @@
 """Used to compile user-friendly parameters into low-level device parameters."""
 
 from caqtus.types.units.unit_namespace import units
+from . import lane_compilation, timed_instructions, timing
 from ._device_compiler import DeviceCompiler, DeviceNotUsedException
 from .compilation_contexts import ShotContext, SequenceContext
-from .variable_namespace import VariableNamespace
 
 __all__ = [
-    "VariableNamespace",
     "units",
     "ShotContext",
     "SequenceContext",
     "DeviceCompiler",
     "DeviceNotUsedException",
+    "lane_compilation",
+    "timed_instructions",
+    "timing",
 ]
