@@ -3,7 +3,7 @@ from __future__ import annotations
 import functools
 from collections.abc import Iterable, Callable, Generator
 from collections.abc import Mapping, Iterator
-from typing import TypeAlias, TypeGuard, Any, assert_type, override, assert_never
+from typing import TypeAlias, TypeGuard, Any, assert_type, override, assert_never, Self
 
 import attrs
 import numpy
@@ -111,7 +111,7 @@ class LinspaceLoop(ContainsSubSteps):
     )
 
     def __str__(self):
-        return f"linspace loop over {fmt.shot_param(self.variable)}"
+        return f"linspace loop over {self.variable}"
 
     def loop_values(
         self, evaluation_context: Mapping[DottedVariableName, Any]
@@ -305,7 +305,8 @@ class ArangeLoop(ContainsSubSteps):
 class ExecuteShot:
     """Step that represents the execution of a shot."""
 
-    pass
+    def __str__(self):
+        return "do shot"
 
 
 def unstructure_hook(execute_shot: ExecuteShot):
@@ -352,6 +353,10 @@ class StepsConfiguration(IterationConfiguration):
         on_setattr=attrs.setters.validate,
     )
 
+    @classmethod
+    def empty(cls) -> Self:
+        return cls(steps=[])
+
     def expected_number_shots(self) -> int | Unknown:
         """Returns the expected number of shots that will be executed by the sequence.
 
