@@ -2,7 +2,7 @@ import attrs
 
 from caqtus.types.image import ImageLabel
 from caqtus.utils import serialization
-from .timelane import TimeLane
+from .timelane import TimeLane, Span
 
 
 @attrs.define
@@ -12,7 +12,12 @@ class TakePicture:
 
 @attrs.define(init=False, eq=False, repr=False)
 class CameraTimeLane(TimeLane[TakePicture | None]):
-    pass
+    def get_picture_labels(self) -> list[ImageLabel]:
+        return [
+            picture.picture_name
+            for picture in self.block_values()
+            if isinstance(picture, TakePicture)
+        ]
 
 
 def unstructure_hook(lane: CameraTimeLane):
@@ -25,7 +30,7 @@ def unstructure_hook(lane: CameraTimeLane):
 
 def structure_hook(data, _) -> CameraTimeLane:
     structured = serialization.structure(
-        data["spanned_values"], list[tuple[TakePicture | None, int]]
+        data["spanned_values"], list[tuple[TakePicture | None, Span]]
     )
     return CameraTimeLane.from_spanned_values(structured)
 
