@@ -1,12 +1,11 @@
 __version__ = "0.1.0"
 
 from . import base
-from .unit_namespace import units
+from ._unit_list import SECOND, NANOSECOND, DECIBEL, MEGAHERTZ, VOLT, HERTZ, AMPERE
 from ._units import (
     ureg,
     unit_registry,
     Quantity,
-    Unit,
     UndefinedUnitError,
     DimensionalityError,
     dimensionless,
@@ -17,16 +16,24 @@ from ._units import (
     CURRENT_UNITS,
     VOLTAGE_UNITS,
     UNITS,
-    UnitLike,
     InvalidDimensionalityError,
+    is_quantity,
+    is_scalar_quantity,
+    Magnitude,
+    Unit,
+    BaseUnit,
+    UnitLike,
 )
+from .unit_namespace import units
 
 __all__ = [
     "__version__",
     "ureg",
     "unit_registry",
     "Quantity",
+    "Magnitude",
     "Unit",
+    "BaseUnit",
     "UndefinedUnitError",
     "DimensionalityError",
     "dimensionless",
@@ -41,4 +48,13 @@ __all__ = [
     "UnitLike",
     "InvalidDimensionalityError",
     "base",
+    "is_quantity",
+    "is_scalar_quantity",
+    "SECOND",
+    "NANOSECOND",
+    "DECIBEL",
+    "MEGAHERTZ",
+    "VOLT",
+    "HERTZ",
+    "AMPERE",
 ]
