@@ -1,5 +1,6 @@
 """Contains utility functions that are not-specific to the package functionalities."""
 
+from . import result
 from ._add_exc_note import add_exc_note
 from ._log_duration import log_duration
 from ._log_exception import log_exception
@@ -11,4 +12,5 @@ __all__ = [
     "DurationTimer",
     "DurationTimerLog",
     "add_exc_note",
+    "result",
 ]
