@@ -9,4 +9,4 @@ def pairwise(iterable: Iterable[T]) -> Iterable[tuple[T, T]]:
     """s -> (s0,s1), (s1,s2), (s2, s3), ..."""
     a, b = tee(iterable)
     next(b, None)
-    return zip(a, b)
+    return zip(a, b, strict=False)
